@@ -4,7 +4,16 @@
 Group <- R6::R6Class("Group",
     inherit=ResultsElement,
     private=list(
-        .items=NA),
+        .items=NA,
+        deep_clone=function(name, value) {
+            if (name == '.items') {
+                items <- list()
+                for (name in names(value))
+                    items[[name]] <- value[[name]]$clone(deep=TRUE)
+                return(items)
+            }
+            value
+        }),
     active=list(
         items=function() private$.items,
         itemNames=function() names(private$.items),
@@ -36,15 +45,24 @@ Group <- R6::R6Class("Group",
             private$.items[[name]]
         },
         .render=function(...) {
+            rendered <- FALSE
             if (self$visible) {
                 for (item in private$.items)
-                    item$.render(...)
+                    rendered <- item$.render(...) || rendered
             }
+            rendered
         },
         add=function(item) {
             item$.parent = self
             private$.items[[item$name]] <- item
         },
+        isFilled=function() {
+            for (item in private$.items) {
+                if (item$visible && item$isNotFilled())
+                    return(FALSE)
+            }
+            TRUE
+        },
         .update=function() {
             if (private$.updated)
                 return()
@@ -96,10 +114,8 @@ Group <- R6::R6Class("Group",
             if ( ! is.null(prepend))
                 group$add("elements", prepend)
 
-            for (item in private$.items) {
-                if (item$visible)
-                    group$add("elements", item$asProtoBuf(incAsText=incAsText, status=status))
-            }
+            for (item in private$.items)
+                group$add("elements", item$asProtoBuf(incAsText=incAsText, status=status))
 
             result <- super$asProtoBuf(incAsText=incAsText, status=status)
             result$group <- group
